@@ -4,14 +4,14 @@ from __future__ import division
 
 import numpy as np
 
+from pandas.compat import lmap, lrange, range, zip
 from pandas.util._decorators import deprecate_kwarg
+
 from pandas.core.dtypes.missing import notna
-from pandas.compat import range, lrange, lmap, zip
+
 from pandas.io.formats.printing import pprint_thing
-
-
 from pandas.plotting._style import _get_standard_colors
-from pandas.plotting._tools import _subplots, _set_ticks_props
+from pandas.plotting._tools import _set_ticks_props, _subplots
 
 
 def scatter_matrix(frame, alpha=0.5, figsize=None, ax=None, grid=False,
@@ -49,7 +49,7 @@ def scatter_matrix(frame, alpha=0.5, figsize=None, ax=None, grid=False,
 
     Examples
     --------
-    >>> df = DataFrame(np.random.randn(1000, 4), columns=['A','B','C','D'])
+    >>> df = pd.DataFrame(np.random.randn(1000, 4), columns=['A','B','C','D'])
     >>> scatter_matrix(df, alpha=0.2)
     """
 
@@ -138,9 +138,6 @@ def scatter_matrix(frame, alpha=0.5, figsize=None, ax=None, grid=False,
 
 def _get_marker_compat(marker):
     import matplotlib.lines as mlines
-    import matplotlib as mpl
-    if mpl.__version__ < '1.1.0' and marker == '.':
-        return 'o'
     if marker not in mlines.lineMarkers:
         return 'o'
     return marker
@@ -185,7 +182,7 @@ def radviz(frame, class_column, ax=None, color=None, colormap=None, **kwds):
 
     See Also
     --------
-    pandas.plotting.andrews_curves : Plot clustering visualization
+    pandas.plotting.andrews_curves : Plot clustering visualization.
 
     Examples
     --------
@@ -206,7 +203,7 @@ def radviz(frame, class_column, ax=None, color=None, colormap=None, **kwds):
         ...                      'versicolor', 'setosa', 'virginica',
         ...                      'setosa']
         ...     })
-        >>> rad_viz = pd.plotting.radviz(df, 'Category')
+        >>> rad_viz = pd.plotting.radviz(df, 'Category')  # doctest: +SKIP
     """
     import matplotlib.pyplot as plt
     import matplotlib.patches as patches
@@ -295,17 +292,17 @@ def andrews_curves(frame, class_column, ax=None, samples=200, color=None,
     class_column : Name of the column containing class names
     ax : matplotlib axes object, default None
     samples : Number of points to plot in each curve
-    color: list or tuple, optional
+    color : list or tuple, optional
         Colors to use for the different classes
     colormap : str or matplotlib colormap object, default None
         Colormap to select colors from. If string, load colormap with that name
         from matplotlib.
-    kwds: keywords
+    kwds : keywords
         Options to pass to matplotlib plotting method
 
     Returns
     -------
-    ax: Matplotlib axis object
+    ax : Matplotlib axis object
 
     """
     from math import sqrt, pi
@@ -338,7 +335,7 @@ def andrews_curves(frame, class_column, ax=None, samples=200, color=None,
     classes = frame[class_column].drop_duplicates()
     df = frame.drop(class_column, axis=1)
     t = np.linspace(-pi, pi, samples)
-    used_legends = set([])
+    used_legends = set()
 
     color_values = _get_standard_colors(num_colors=len(classes),
                                         colormap=colormap, color_type='random',
@@ -406,9 +403,8 @@ def bootstrap_plot(series, fig=None, size=50, samples=500, **kwds):
     .. plot::
             :context: close-figs
 
-            >>> import numpy as np
             >>> s = pd.Series(np.random.uniform(size=100))
-            >>> fig = pd.plotting.bootstrap_plot(s)
+            >>> fig = pd.plotting.bootstrap_plot(s)  # doctest: +SKIP
     """
     import random
     import matplotlib.pyplot as plt
@@ -465,31 +461,31 @@ def parallel_coordinates(frame, class_column, cols=None, ax=None, color=None,
 
     Parameters
     ----------
-    frame: DataFrame
-    class_column: str
+    frame : DataFrame
+    class_column : str
         Column name containing class names
-    cols: list, optional
+    cols : list, optional
         A list of column names to use
-    ax: matplotlib.axis, optional
+    ax : matplotlib.axis, optional
         matplotlib axis object
-    color: list or tuple, optional
+    color : list or tuple, optional
         Colors to use for the different classes
-    use_columns: bool, optional
+    use_columns : bool, optional
         If true, columns will be used as xticks
-    xticks: list or tuple, optional
+    xticks : list or tuple, optional
         A list of values to use for xticks
-    colormap: str or matplotlib colormap, default None
+    colormap : str or matplotlib colormap, default None
         Colormap to use for line colors.
-    axvlines: bool, optional
+    axvlines : bool, optional
         If true, vertical lines will be added at each xtick
-    axvlines_kwds: keywords, optional
+    axvlines_kwds : keywords, optional
         Options to be passed to axvline method for vertical lines
-    sort_labels: bool, False
+    sort_labels : bool, False
         Sort class_column labels, useful when assigning colors
 
         .. versionadded:: 0.20.0
 
-    kwds: keywords
+    kwds : keywords
         Options to pass to matplotlib plotting method
 
     Returns
@@ -498,13 +494,12 @@ def parallel_coordinates(frame, class_column, cols=None, ax=None, color=None,
 
     Examples
     --------
-    >>> from pandas import read_csv
-    >>> from pandas.tools.plotting import parallel_coordinates
     >>> from matplotlib import pyplot as plt
-    >>> df = read_csv('https://raw.github.com/pandas-dev/pandas/master'
-                      '/pandas/tests/data/iris.csv')
-    >>> parallel_coordinates(df, 'Name', color=('#556270',
-                             '#4ECDC4', '#C7F464'))
+    >>> df = pd.read_csv('https://raw.github.com/pandas-dev/pandas/master'
+                        '/pandas/tests/data/iris.csv')
+    >>> pd.plotting.parallel_coordinates(
+            df, 'Name',
+            color=('#556270', '#4ECDC4', '#C7F464'))
     >>> plt.show()
     """
     if axvlines_kwds is None:
@@ -520,7 +515,7 @@ def parallel_coordinates(frame, class_column, cols=None, ax=None, color=None,
     else:
         df = frame[cols]
 
-    used_legends = set([])
+    used_legends = set()
 
     ncols = len(df.columns)
 
@@ -577,10 +572,10 @@ def lag_plot(series, lag=1, ax=None, **kwds):
 
     Parameters
     ----------
-    series: Time series
-    lag: lag of the scatter plot, default 1
-    ax: Matplotlib axis object, optional
-    kwds: Matplotlib scatter method keyword arguments, optional
+    series : Time series
+    lag : lag of the scatter plot, default 1
+    ax : Matplotlib axis object, optional
+    kwds : Matplotlib scatter method keyword arguments, optional
 
     Returns
     -------
